from sinta_scraper.affil_scraper import affil, affil_authors
from sinta_scraper.author_scraper import author
from sinta_scraper.comm_svc_scraper import author_comm_services, dept_comm_services
from sinta_scraper.dept_scraper import dept_authors
from sinta_scraper.ipr_scraper import author_ipr, dept_ipr
from sinta_scraper.research_scraper import author_researches, dept_researches
from sinta_scraper.scholar_documents_scraper import author_scholar
from sinta_scraper.scopus_documents_scraper import author_scopus, author_scopus_journal, \
    author_scopus_conference, dept_scopus
from sinta_scraper.wos_documents_scraper import author_wos, dept_wos



import re
from concurrent.futures import ThreadPoolExecutor

from bs4 import BeautifulSoup
from requests import get

import utils
from utils.config import get_config


def author(author_ids, output_format='dictionary'):
    if type(author_ids) is not list and type(author_ids) is not tuple:
        author_ids = [author_ids]

    worker_result = []

    with ThreadPoolExecutor() as executor:
        for author_id in author_ids:
            executor.submit(worker, author_id, worker_result)

    if len(worker_result) == 1:
        worker_result = worker_result[0]

    return utils.format_output(worker_result, output_format)


def worker(author_id, worker_result):
    domain = get_config()['domain']
    url = f'{domain}/authors/detail?id={author_id}&view=overview'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')

    name = soup.select('.au-name')[0].text.strip()
    areas = [area.text.strip() for area in soup.select('.area-item')]
    scores_soup = soup.select('.stat2-val')

    index_score_names = ['documents', 'citations', 'h-index', 'i10-index', 'g-index']
    index_scores = soup.select('.stat-num-pub')

    scopus = {index_score_names[i]: utils.cast(index_scores[i + 16].text) for i in range(len(index_score_names))}
    scopus_outputs_names = ['articles', 'conferences', 'others']
    scopus_outputs = {scopus_outputs_names[i]: utils.cast(soup.select('.stat-num-pub')[i].text) for i in range(3)}
    scopus_quartiles = {f'Q{i}': utils.cast(soup.select('.stat-num-pub')[i + 3].text) for i in range(1, 5)}
    scopus = {**scopus, **scopus_outputs, **scopus_quartiles}
    scopus['undefined'] = utils.cast(soup.select('.stat-num-pub')[8].text)

    scholar = {index_score_names[i]: utils.cast(index_scores[i + 21].text) for i in range(len(index_score_names))}
    wos = {index_score_names[i]: utils.cast(index_scores[i + 26].text) for i in range(len(index_score_names))}

    sinta = {f'S{i}': utils.cast(index_scores[i + 9].text) for i in range(6)}
    sinta['uncategorized'] = utils.cast(index_scores[15].text)

    score_names = ['overall', '3_years', 'overall_v2', '3_years_v2']
    scores = {score_name: float(scores_soup[i].text) for i, score_name in enumerate(score_names)}

    books = utils.cast(scores_soup[4].text)
    ipr = utils.cast(scores_soup[7].text)

    rank_names = ['national', '3_years_national', 'ipr', 'affiliation', '3_years_affiliation']
    ranks = {rank_names[i]: utils.cast(scores_soup[i + 5].text) for i in [0, 1, 3, 4]}

    affil = soup.select('.au-affil > a')
    dept = soup.select('.au-department')[0].text.strip()
    affil_name = affil[0].text.strip()
    affil_url = 'http://sinta.ristekbrin.go.id' + affil[0]['href']
    affil_id = re.search(r'id=(\d+)', affil_url).group(1)

    result_data = {
        'id': author_id,
        'name': name,
        'url': url,
        'affiliation': {
            'id': affil_id,
            'name': affil_name,
            'url': affil_url
        },
        'department': dept,
        'areas': areas,
        'score': scores,
        'rank': ranks,
        'scopus': scopus,
        'scholar': scholar,
        'wos': wos,
        'sinta': sinta,
        'books': books,
        'ipr': ipr
    }

    worker_result.append(result_data)



from concurrent.futures import ThreadPoolExecutor

from bs4 import BeautifulSoup
from requests import get

from sinta_scraper.dept_scraper import dept_authors
from utils.config import get_config
from utils.utils import cast, format_output, listify


def author_wos(author_id, output_format='dictionary'):
    domain = get_config()['domain']
    url = f'{domain}/authors/detail?id={author_id}&view=documentswos'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')
    page_info = soup.select('.uk-width-large-1-2.table-footer')
    n_page = cast(page_info[0].text.strip().split()[3])
    worker_result = parse(soup)

    with ThreadPoolExecutor() as executor:
        for page in range(2, n_page + 1):
            executor.submit(worker, author_id, page, worker_result)

    return format_output(worker_result, output_format)


def worker(author_id, page, worker_result):
    domain = get_config()['domain']
    url = f'{domain}/authors/detail?page={page}&id={author_id}&view=documentswos'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')
    data = parse(soup)

    worker_result.extend(data)


def parse(soup):
    rows = soup.select('table.uk-table tr')
    result = []

    for row in rows:
        link = row.select('a.paper-link')

        if not link:
            continue

        link = link[0]
        info1 = row.select('dd.indexed-by')[0].text.strip().split('|')
        info2 = row.select('dd')[0].text.strip().split('\n')
        issn = info2[1].split(':')[1].strip()
        doi = info2[2].split(':')[1].strip()
        uid = info2[3].split(' : ')[1].strip()

        result.append({
            'title': link.text,
            'publisher': info1[0].strip(),
            'issn': issn if issn else '-',
            'doi': doi if doi else '-',
            'uid': uid if uid else '-'
        })

    return result


def dept_wos(dept_ids, affil_id, output_format='dictionary'):
    dept_ids = listify(dept_ids)

    authors = []
    worker_result = []

    for dept_id in dept_ids:
        authors.extend(dept_authors(dept_id, affil_id))

    with ThreadPoolExecutor() as executor:
        for author in authors:
            executor.submit(dept_wos_docs_worker, author['id'], worker_result)

    return format_output(worker_result, output_format)


def dept_wos_docs_worker(author_id, worker_result):
    wos_docs = author_wos(author_id)

    worker_result.extend(wos_docs)



import re
from concurrent.futures import ThreadPoolExecutor

from bs4 import BeautifulSoup
from requests import get
from string_utils.validation import is_integer

from utils.config import get_config
from utils.utils import cast, format_output, listify


def author_scholar(author_id, output_format='dictionary', min_year=None, max_year=None):
    domain = get_config()['domain']
    url = f'{domain}/authors/detail?id={author_id}&view=documentsgs'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')
    page_info = soup.select('.uk-width-large-1-2.table-footer')
    n_page = cast(page_info[0].text.strip().split()[3])
    worker_result = author_scholar_parser(soup)

    with ThreadPoolExecutor() as executor:
        for page in range(2, n_page + 1):
            executor.submit(author_scholar_worker, author_id, page, worker_result)

    if min_year is not None:
        worker_result = [doc for doc in worker_result if is_integer(str(doc['year'])) and doc['year'] >= min_year]

    if max_year is not None:
        worker_result = [doc for doc in worker_result if is_integer(str(doc['year'])) and doc['year'] <= max_year]

    return format_output(worker_result, output_format)


def author_scholar_worker(author_id, page, worker_result):
    domain = get_config()['domain']
    url = f'{domain}/authors/detail?page={page}&id={author_id}&view=documentsgs'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')
    data = author_scholar_parser(soup)

    worker_result.extend(data)


def author_scholar_parser(soup):
    rows = soup.select('table.uk-table tr')
    result = []
    publisher_regex = re.compile(r'([A-Za-z ]+)\s*(\d+)\s*\((\d+)\)[,\s]*(\d+-*\d*)[,\s]*(\d{4})*')

    for row in rows:
        link = row.select('a.paper-link')

        if not link:
            continue

        link = link[0]
        authors = row.select('dd')[0].text.split(', ')
        info = row.select('dd.indexed-by')[0].text.strip().split('|')
        citations = row.select('.index-val')[1].text.strip()
        publisher_full = info[0].strip()
        publisher_parsed = publisher_regex.search(publisher_full)

        if publisher_parsed:
            fields = 'full', 'name', 'volume', 'issue', 'pages', 'year'
            publisher = {field: cast(publisher_parsed.group(i)) for i, field in enumerate(fields)}
        else:
            publisher = publisher_full

        result.append({
            'title': link.text,
            'authors': authors,
            'url': link['href'],
            'publisher': publisher,
            'year': cast(info[3].strip()),
            'citations': cast(citations) if is_integer(citations) else 0
        })

    return result


def dept_scholar(dept_ids, affil_id, output_format='dictionary', min_year=None, max_year=None):
    dept_ids = listify(dept_ids)
    worker_result = []

    for dept_id in dept_ids:
        domain = get_config()['domain']
        url = f'{domain}/departments/detail?afil={affil_id}&id={dept_id}&view=documents'
        html = get(url)
        soup = BeautifulSoup(html.content, 'html.parser')
        page_info = soup.select('.uk-width-large-1-2.table-footer')
        n_page = cast(page_info[0].text.strip().split()[3])

        worker_result.extend(dept_scholar_parser(soup, min_year, max_year))

        with ThreadPoolExecutor() as executor:
            for page in range(2, n_page + 1):
                executor.submit(dept_scholar_worker, dept_id, affil_id, page, min_year, max_year, worker_result)

    return format_output(worker_result, output_format)


def dept_scholar_worker(dept_id, affil_id, page, min_year, max_year, worker_result):
    domain = get_config()['domain']
    url = f'{domain}/departments/detail?page={page}&id={dept_id}&afil={affil_id}&view=documents'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')
    result = dept_scholar_parser(soup, min_year, max_year)

    worker_result.extend(result)


def dept_scholar_parser(soup, min_year, max_year):
    rows = soup.select('table.uk-table tr')
    result = []
    publisher_regex = re.compile(r'([A-Za-z ]+)\s*(\d+)\s*\((\d+)\)[,\s]*(\d+-*\d*)[,\s]*(\d{4})*')

    for row in rows:
        link = row.select('a.paper-link')

        if not link:
            continue

        link = link[0]
        authors = row.select('dd')[0].text.split(', ')
        info = row.select('dd.indexed-by')[0].text.strip().split('|')
        citations = row.select('.index-val')[1].text.strip()
        year = cast(info[3].strip())
        publisher_full = info[0].strip()
        publisher_parsed = publisher_regex.search(publisher_full)

        if publisher_parsed:
            fields = 'full', 'name', 'volume', 'issue', 'pages', 'year'
            publisher = {field: cast(publisher_parsed.group(i)) for i, field in enumerate(fields)}
        else:
            publisher = publisher_full

        if (min_year is not None and is_integer(str(year)) and int(year) < min_year) or (
                max_year is not None and is_integer(str(year)) and int(year) > min_year):
            continue

        result.append({
            'title': link.text,
            'authors': authors,
            'url': link['href'],
            'publisher': publisher,
            'year': cast(info[3].strip()),
            'citations': cast(citations) if is_integer(citations) else 0
        })

    return result



import json

from dict2xml import dict2xml
from string_utils.validation import is_integer, is_decimal


def format_output(obj, output_format):
    if output_format == 'json':
        return json.dumps(obj)
    elif output_format == 'json-pretty':
        return json.dumps(obj, indent=4)
    elif output_format == 'xml':
        return dict2xml(obj, wrap='author')
    elif output_format == 'xml-pretty':
        return dict2xml(obj, wrap='author', indent='    ')
    else:
        return obj


def cast(string: str):
    if string is None:
        return None

    string = string.strip()

    if is_integer(string):
        return int(string)
    elif is_decimal(string):
        return float(string)
    elif string == '-':
        return None
    else:
        return string


def listify(param):
    return [param] if type(param) not in [list, tuple] else param



import re
from concurrent.futures import ThreadPoolExecutor

from bs4 import BeautifulSoup
from requests import get

import utils
from sinta_scraper.dept_scraper import dept_authors
from utils.config import get_config


def author_researches(author_id, output_format='dictionary'):
    domain = get_config()['domain']
    url = f'{domain}/authors/detail?id={author_id}&view=research'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')
    page_info = soup.select('.uk-width-large-1-2.table-footer')
    n_page = utils.cast(page_info[0].text.strip().split()[3])
    worker_result = parse(soup)

    with ThreadPoolExecutor() as executor:
        for page in range(2, n_page + 1):
            executor.submit(author_researches_worker, author_id, page, worker_result)

    return utils.format_output(worker_result, output_format)


def author_researches_worker(author_id, page, worker_result):
    domain = get_config()['domain']
    url = f'{domain}/authors/detail?page={page}&id={author_id}&view=research'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')
    data = parse(soup)

    worker_result.extend(data)


def parse(soup):
    rows = soup.select('table.uk-table tr')
    result = []

    for row in rows:
        link = row.select('a.paper-link')

        if not link:
            continue

        link = link[0]
        info1 = row.select('dd.indexed-by-orange')[0].text.split('|')
        dd = row.select('dd')
        info2 = [i.split(':')[1].strip() for i in dd[2].text.strip().split('\r\n')]
        members = [member.strip() for member in dd[1].text.split(',') if member.strip()]

        result.append({
            'title': link.text.strip(),
            'scheme': info1[0].split(':')[1].strip(),
            'source': info1[1].split(':')[1].strip(),
            'members': members,
            'application_year': utils.cast(info2[0]),
            'event_year': utils.cast(info2[1]),
            'fund': utils.cast(re.sub(r'[Rp\.\s\,]', '', info2[2])[:-2]),
            'field': dd[3].text.strip(),
            'sponsor': row.select('td.uk-text-center')[0].text.strip()
        })

    return result


def dept_researches(dept_ids, affil_id, output_format='dictionary'):
    if type(dept_ids) is not list and type(dept_ids) is not tuple:
        dept_ids = [dept_ids]

    authors = []
    worker_result = []

    for dept_id in dept_ids:
        authors.extend(dept_authors(dept_id, affil_id))

    with ThreadPoolExecutor() as executor:
        for author in authors:
            executor.submit(dept_researches_worker, author['id'], worker_result)

    return utils.format_output(worker_result, output_format)


def dept_researches_worker(author_id, worker_result):
    researches = author_researches(author_id)

    worker_result.extend(researches)



import re
from concurrent.futures import ThreadPoolExecutor

from bs4 import BeautifulSoup
from requests import get

from utils.config import get_config
from utils.utils import format_output, cast, listify


def retrieve_authors(dept_id, affil_id, output_format='dictionary'):
    domain = get_config()['domain']
    url = f'{domain}/departments/detail?afil={affil_id}&id={dept_id}&view=authors'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')
    page_info = soup.select('.uk-width-large-1-2.table-footer')
    n_page = cast(page_info[0].text.strip().split()[3])
    worker_result = author_parser(soup)

    with ThreadPoolExecutor() as executor:
        for page in range(2, n_page + 1):
            executor.submit(retrieve_authors_worker, dept_id, affil_id, page, worker_result)

    return format_output(worker_result, output_format)


def retrieve_authors_worker(dept_id, affil_id, page, worker_result):
    domain = get_config()['domain']
    url = f'{domain}/departments/detail?page={page}&afil={affil_id}&id={dept_id}&view=authors&sort=year2'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')
    data = author_parser(soup)

    worker_result.extend(data)


def author_parser(soup):
    links = soup.select('.uk-description-list-line .text-blue')
    result = []

    for i in range(len(links)):
        link = links[i]
        author_id = re.search(r'id=(\d+)', link['href']).group(1)
        author_name = link.text

        result.append({
            'id': author_id,
            'name': author_name.title()
        })

    return result


def dept_authors(dept_ids, affil_id, output_format='dictionary'):
    dept_ids = listify(dept_ids)
    worker_result = []

    with ThreadPoolExecutor() as executor:
        for dept_id in dept_ids:
            executor.submit(dept_authors_worker, dept_id, affil_id, worker_result)

    return format_output(worker_result, output_format)


def dept_authors_worker(dept_id, affil_id, worker_result):
    authors = retrieve_authors(dept_id, affil_id)

    worker_result.extend(authors)



import re
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime

from bs4 import BeautifulSoup
from requests import get
from string_utils.validation import is_integer

from sinta_scraper.dept_scraper import dept_authors
from utils.config import get_config
from utils.utils import cast, format_output, listify


def author_scopus(author_id, output_format='dictionary', min_date=None, max_date=None):
    domain = get_config()['domain']
    url = f'{domain}/authors/detail?id={author_id}&view=documentsscopus'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')
    page_info = soup.select('.uk-width-large-1-2.table-footer')
    n_page = cast(page_info[0].text.strip().split()[3])
    worker_result = parse(soup)
    date_format = '%Y-%m-%d'

    with ThreadPoolExecutor() as executor:
        for page in range(2, n_page + 1):
            executor.submit(worker, author_id, page, worker_result)

    if min_date is not None:
        min_date_parsed = datetime.strptime(min_date, date_format)
        worker_result = [doc for doc in worker_result if datetime.strptime(doc['date'], date_format) >= min_date_parsed]

    if max_date is not None:
        max_date_parsed = datetime.strptime(max_date, date_format)
        worker_result = [doc for doc in worker_result if datetime.strptime(doc['date'], date_format) <= max_date_parsed]

    return format_output(worker_result, output_format)


def author_scopus_journal(author_id, output_format='dictionary', min_date=None, max_date=None):
    docs = author_scopus(author_id, min_date=min_date, max_date=max_date)
    journal_docs = [doc for doc in docs if doc['type'] == 'Journal']

    return format_output(journal_docs, output_format)


def author_scopus_conference(author_id, output_format='dictionary', min_date=None, max_date=None):
    docs = author_scopus(author_id, min_date=min_date, max_date=max_date)
    journal_docs = [doc for doc in docs if doc['type'].startswith('Conference')]

    return format_output(journal_docs, output_format)


def worker(author_id, page, worker_result):
    domain = get_config()['domain']
    url = f'{domain}/authors/detail?page={page}&id={author_id}&view=documentsscopus'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')
    data = parse(soup)

    worker_result.extend(data)


def parse(soup):
    rows = soup.select('table.uk-table tr')
    result = []

    for row in rows:
        link = row.select('a.paper-link')

        if not link:
            continue

        link = link[0]
        info1 = row.select('.index-val')
        quartile = info1[0].text.strip()
        citations = info1[1].text.strip()
        info2 = row.select('dd.indexed-by')[0].text.strip().split('|')

        result.append({
            'title': link.text,
            'url': link['href'],
            'publisher': info2[0].strip(),
            'date': info2[3].strip(),
            'type': info2[4].strip(),
            'quartile': cast(quartile[1]) if re.search(r'^Q[1-4]{1}$', quartile) else '-',
            'citations': cast(citations) if is_integer(citations) else 0
        })

    return result


def dept_scopus(dept_ids, affil_id, output_format='dictionary', max_workers=None):
    dept_ids = listify(dept_ids)

    authors = []
    worker_result = []

    for dept_id in dept_ids:
        authors.extend(dept_authors(dept_id, affil_id))

    with ThreadPoolExecutor(max_workers=max_workers) as executor:
        for author in authors:
            executor.submit(dept_scopus_docs_worker, author['id'], worker_result)

    return format_output(worker_result, output_format)


def dept_scopus_docs_worker(author_id, worker_result):
    researches = author_scopus(author_id)

    worker_result.extend(researches)



import re
from concurrent.futures import ThreadPoolExecutor

from bs4 import BeautifulSoup
from requests import get

import utils
from utils.config import get_config


def affil(affil_ids, output_format='dictionary'):
    if type(affil_ids) is not list and type(affil_ids) is not tuple:
        affil_ids = [affil_ids]

    worker_result = []

    with ThreadPoolExecutor() as executor:
        for affil_id in affil_ids:
            executor.submit(affil_worker, affil_id, worker_result)

    if len(worker_result) == 1:
        worker_result = worker_result[0]

    return utils.format_output(worker_result, output_format)


def affil_worker(affil_id, worker_result):
    domain = get_config()['domain']
    url = f'{domain}/affiliations/detail?id={affil_id}&view=overview'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')

    name = soup.select('.au-name')[0].text.strip()
    affil_url = soup.select('.au-department > a')[0].text.strip()

    stats = [utils.cast(soup.select('.stat2-val')[i].text.strip().replace(',', '')) for i in range(9)]

    result_data = {
        'name': name,
        'url': affil_url,
        'score': {
            'overall': stats[0],
            'overall_v2': stats[1],
            '3_years': stats[3],
            '3_years_v2': stats[4]
        },
        'rank': {
            'national': stats[2],
            '3_years_national': stats[5]
        },
        'journals': stats[6],
        'verified_authors': stats[7],
        'lecturers': stats[8]
    }

    worker_result.append(result_data)


def affil_authors(affil_id, output_format='dictionary'):
    domain = get_config()['domain']
    url = f'{domain}/affiliations/detail?id={affil_id}&view=authors'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')
    page_info = soup.select('.uk-width-large-1-2.table-footer')
    n_page = utils.cast(page_info[0].text.strip().split()[3])
    worker_result = author_parser(soup)

    with ThreadPoolExecutor() as executor:
        for page in range(2, n_page + 1):
            executor.submit(affil_authors_page_worker, affil_id, page, worker_result)

    return utils.format_output(worker_result, output_format)


def affil_authors_page_worker(affil_id, page, worker_result):
    domain = get_config()['domain']
    url = f'{domain}/affiliations/detail?page={page}&view=authors&id={affil_id}&sort=year2'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')
    data = author_parser(soup)

    worker_result.extend(data)


def author_parser(soup):
    rows = soup.select('.uk-description-list-line')
    result = []

    for row in rows:
        link = row.select('.text-blue')[0]
        author_id = re.search(r'id=(\d+)', link['href'].strip()).group(1)
        author_name = link.text.strip()
        nidn = row.select('dd')[1].text.split()[-1]

        result.append({
            'id': author_id,
            'name': author_name.title(),
            'nidn': nidn
        })

    return result
